import 'package:flutter/material.dart';
import 'package:dotted_border/dotted_border.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

enum SeniorSquareButtonsMenuItemType {
  primary,
  secondary,
  ghost,
}

class SeniorSquareButtonsMenuItemData {
  final IconData icon;
  final String text;
  final SeniorSquareButtonsMenuItemType type;
  final Function() onTap;

  SeniorSquareButtonsMenuItemData({
    required this.icon,
    required this.text,
    required this.type,
    required this.onTap,
  });
}

class SeniorSquareButtonsMenu extends StatelessWidget {
  final List<SeniorSquareButtonsMenuItemData> items;

  const SeniorSquareButtonsMenu({
    Key? key,
    required this.items,
  }) : super(key: key);

  factory SeniorSquareButtonsMenu.grid({
    Key? key,
    required List<SeniorSquareButtonsMenuItemData> items,
    int crossAxisCount,
  }) = _SeniorSquadButtonsMenuGrid;

  Color _getBackgroundColor(SeniorSquareButtonsMenuItemType type) {
    switch (type) {
      case SeniorSquareButtonsMenuItemType.primary:
        return SeniorColors.primaryColor;
      case SeniorSquareButtonsMenuItemType.secondary:
        return SeniorColors.secondaryColor600;
      case SeniorSquareButtonsMenuItemType.ghost:
        return Colors.transparent;
    }
  }

  Color _getDrawingColor(SeniorSquareButtonsMenuItemType type) {
    switch (type) {
      case SeniorSquareButtonsMenuItemType.ghost:
        return SeniorColors.secondaryColor600;
      default:
        return SeniorColors.pureWhite;
    }
  }

  Widget buildMenuItem({
    required SeniorSquareButtonsMenuItemData item,
    required double itemsSize,
  }) {
    final double iconSize = itemsSize * 0.3;

    return Padding(
      padding: const EdgeInsets.all(4.0),
      child: Tooltip(
        message: item.text,
        child: GestureDetector(
          onTap: item.onTap,
          child: DottedBorder(
            borderType: BorderType.RRect,
            radius: Radius.circular(6.0),
            padding: const EdgeInsets.all(0),
            color: item.type == SeniorSquareButtonsMenuItemType.ghost
                ? SeniorColors.secondaryColor600
                : Colors.transparent,
            child: Container(
              height: itemsSize,
              width: itemsSize,
              padding: EdgeInsets.all(16.0),
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(6.0),
                gradient: item.type == SeniorSquareButtonsMenuItemType.primary
                    ? LinearGradient(
                        begin: Alignment.bottomLeft,
                        end: Alignment.topRight,
                        colors: SeniorColors.primaryGradientColors,
                      )
                    : null,
                color: _getBackgroundColor(item.type),
              ),
              child: Column(
                mainAxisAlignment: itemsSize >= 100.0
                    ? MainAxisAlignment.spaceBetween
                    : MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  Icon(
                    item.icon,
                    color: _getDrawingColor(item.type),
                    size: iconSize,
                  ),
                  Visibility(
                    visible: itemsSize >= 100.0,
                    child: Text(
                      item.text,
                      textAlign: TextAlign.center,
                      overflow: TextOverflow.ellipsis,
                      maxLines: 2,
                      style: SeniorTypography.small(
                        color: _getDrawingColor(item.type),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      scrollDirection: Axis.horizontal,
      child: Row(
        children: items
            .map((item) => buildMenuItem(item: item, itemsSize: 100.0))
            .toList(),
      ),
    );
  }
}

class _SeniorSquadButtonsMenuGrid extends SeniorSquareButtonsMenu {
  late final int crossAxisCount;

  _SeniorSquadButtonsMenuGrid({
    Key? key,
    required List<SeniorSquareButtonsMenuItemData> items,
    int crossAxisCount = 3,
  }) : super(key: key, items: items) {
    this.crossAxisCount = crossAxisCount <= 6 ? crossAxisCount : 6;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          double menuWidth = constraints.constrainWidth();
          int slots = items.length;
          while (slots % crossAxisCount != 0) slots++;
          double itemsSize = (menuWidth - 8 * crossAxisCount) / crossAxisCount;
          int linesCount = (items.length / crossAxisCount).ceil();
          double menuHeight = slots / crossAxisCount * itemsSize + linesCount * 8;

          return SizedBox(
            height: menuHeight,
            width: menuWidth,
            child: GridView.count(
              crossAxisCount: crossAxisCount,
              physics: RangeMaintainingScrollPhysics(),
              children: items
                  .map((item) => Container(
                        height: itemsSize,
                        width: itemsSize,
                        child: buildMenuItem(item: item, itemsSize: itemsSize),
                      ))
                  .toList(),
            ),
          );
        },
      ),
    );
  }
}
